<?php 

namespace Teknisa\CNC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;

use Teknisa\CNC\Utilities;

use Teknisa\CNC\Services\Cnc00800CompVendasService;

class Cnc00800CompVendas extends \Teknisa\CNC\Controllers\TeknisaCrud {
	protected $parameterBagColumns;
	protected $util;
	protected $compValoresService;
	
	public function __construct($crudService, ParameterBag $parameterBag, Utilities\Util $util, Cnc00800CompVendasService $compValoresService) {
		$this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->compValoresService = $compValoresService;
	}
	
	private function handleParamMultiValue(&$conditions, $paramName, $chr = '|') {
		if(isset($conditions[$paramName]) && !empty($conditions[$paramName])) {
			if(is_array($conditions[$paramName])) {
				$conditions[$paramName] = '|' . implode($chr, $conditions[$paramName]) . '|';
			} else if(is_string($conditions[$paramName])) {
				$conditions[$paramName] = '|' . $conditions[$paramName] . '|';
			}
		}
	}

	public function find (DTO\Request\Filter $request, DTO\Response $response) {
		try {
	        $row = $request->getFilterCriteria()->getConditions();
	        $conditions = $this->util->getParamsAsArray($request);
	        
	        $this->util->setTCDcondition($conditions, 'NRESTABELEC');
	        $this->util->setTCDcondition($conditions, 'NRADQUIRENTE');
	        $this->util->setTCDcondition($conditions, 'NRPRODUTOADQ');
	        $this->util->setTCDcondition($conditions, 'NRBANDEIRAVAN');

					$this->handleParamMultiValue($conditions, 'NRESTABELEC');
					$this->handleParamMultiValue($conditions, 'NRADQUIRENTE');
					$this->handleParamMultiValue($conditions, 'NRPRODUTOADQ');
					$this->handleParamMultiValue($conditions, 'NRBANDEIRAVAN');
	        
	        if($conditions['PERIODO'] === 'M') {
		        $formatedDates = $this->compValoresService->formatAndValidateMonthRows($conditions);
	        	$conditions['DTVENDAINI'] = $formatedDates[0];
	        	$conditions['DTVENDAFIN'] = $formatedDates[1];
	        } else {
	        	$this->compValoresService->validateDayPeriodRow($conditions);
	        }
	        
	        $dataSet = $this->compValoresService->getValoresVendas($conditions);
	        $response->addDataSet(new DataSet('dataSet', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
	}

	public function getValoresVendasDetails(Request\Filter $request, Response $response){
		$allFieldsLabel = 'DTVENDA|NMESTABELEC|NMADQUIRENTE|CDNSU|VRESPERADO|VRRECEBIDO';
        try {
            $row = $request->getFilterCriteria()->getConditions();
            $conditions = $this->util->getParamsAsArray($request);
            
	        $this->util->setTCDcondition($conditions, 'NRESTABELEC');
	        $this->util->setTCDcondition($conditions, 'NRADQUIRENTE');
	        $this->util->setTCDcondition($conditions, 'NRPRODUTOADQ');
	        $this->util->setTCDcondition($conditions, 'NRBANDEIRAVAN');

					$this->handleParamMultiValue($conditions, 'NRESTABELEC');
					$this->handleParamMultiValue($conditions, 'NRADQUIRENTE');
					$this->handleParamMultiValue($conditions, 'NRPRODUTOADQ');
					$this->handleParamMultiValue($conditions, 'NRBANDEIRAVAN');
	        
	        $this->util->setAllFields($conditions, $allFieldsLabel);
			
	        if($conditions['PERIODO'] === 'M') {
		        $formatedDates = $this->compValoresService->formatAndValidateMonthRows($conditions);
	        	$conditions['DTVENDAINI'] = $formatedDates[0];
	        	$conditions['DTVENDAFIN'] = $formatedDates[1];
	        } else {
	        	$this->compValoresService->validateDayPeriodRow($conditions);
	        }
	        unset($conditions['PERIODO']);
	        
	        $resultadoFiltrado = $this->compValoresService->getValoresVendasDetails($conditions);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc00800_GetComparacaoValoresDetalhes', $resultadoFiltrado));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
}