<?php

namespace WebServiceCall\Repository\DB;

class Payments extends ABaseRepo {
    
    private function getParameters($payment) {
        return array(
            ':NRRETORIGFINANC' => (int)$this->conn->getNextTableId('RETORIGFINANC'),
            ':NRCTRLACESWS' => $payment[':NRCTRLACESWS'],
            ':Estabelecimento' => $payment[':Estabelecimento'],
            ':Adquirente' => $payment[':Adquirente'],
            ':NRADQUIRENTE' => $payment[':NRADQUIRENTE'],
            ':CDPRODUTOADQ' => $payment[':CDPRODUTOADQ'],
            ':TipoProduto' => $payment[':TipoProduto'],
            ':NRPRODUTOADQ' => $payment[':NRPRODUTOADQ'],
            ':DataVenda' => $payment[':DataVenda'],
            ':DataPagamento' => $payment[':DataPagamento'],
            ':ResumoVenda' => $payment[':ResumoVenda'],
            ':Autorizacao' => $payment[':Autorizacao'],
            ':NSU' => $payment[':NSU'],
            ':TID' => $payment[':TID'],
            ':ValorBruto' => $payment[':ValorBruto'],
            ':Taxa' => $payment[':Taxa'],
            ':VRCOBRADOTAXA' => $payment[':VRCOBRADOTAXA'],
            ':VRLIQVENDA' => $payment[':VRLIQVENDA'],
            ':Banco' => $payment[':Banco'],
            ':Agencia' => $payment[':Agencia'],
            ':Conta' => $payment[':Conta'],
            ':Parcela' => $payment[':Parcela'],
            ':TotalParcelas' => $payment[':TotalParcelas'],
            ':Antecipacao' => $payment[':Antecipacao'],
            ':IdTipoTransacao' => $payment[':IdTipoTransacao'],
            ':TipoTransacao' => $payment[':TipoTransacao'],
            ':NRORG' => $payment[':NRORG'],
            ':IDATIVO' => $payment[':IDATIVO'],
            ':DTINCLUSAO' => $payment[':DTINCLUSAO'],
            ':DTULTATU' => $payment[':DTULTATU'],
            ':NRORGINCLUSAO' => $payment[':NRORGINCLUSAO'],
            ':NRORGULTATU' => $payment[':NRORGULTATU']
        );
    }
    
    public function insertPayment($payment) {
        return $this->conn->executeQuery(self::INSERT_PAYMENT, $this->getParameters($payment), [], true);
    }

    /*
     * Verifica se na tabela RETORIGVENDA, já está inserido o registro em questão utilizando a data da venda, o NSU e o 
     * código da autorização, como parâmetros para busca da linha.
     *
     * @param nsu
     * @param autorizacao
     * @param dtvenda
     *
     * @return bool: true se a o select não encontrar nenhuma linha com os dados da filtragem, false caso contrário.
     */
     
    public function isPaymentInserted($nsu, $cdLoteVenda, $autorizacao, $dtCredito, $vrVenda) {
        if (!$nsu && !$cdLoteVenda) {
            $linhas = $this->getPaymentByCDLOTEVENDA($cdLoteVenda, $dtCredito, $vrVenda);
        } else if (!$nsu && !$autorizacao) {
            $linhas = $this->getPaymentByNRAUTORIZACAO($cdLoteVenda, $autorizacao, $dtCredito, $vrVenda);
        } else if ($nsu && !$cdLoteVenda && !$autorizacao) {
            $linhas = $this->getPaymentByNSU($nsu, $dtCredito, $vrVenda);
        } else {
            $linhas = $this->getPaymentDefault($nsu, $autorizacao, $dtCredito);
        }
        
        return (intval($linhas) !== 0);
    }
    
    private function getPaymentDefault($nsu, $autorizacao, $dtCredito) {
         $params = array(
            ":DTCREDITO"      => $dtCredito,
            ":NRORG"          => $this->conn->getNRORG(),
            ":NRAUTORIZACAO"  => $autorizacao,
            ":CDNSU"          => $nsu,
         );
         
         return $this->conn->fetchRow(self::GET_PAYMENT_DEFAULT, $params, [], true);
    }
    
    private function getPaymentByNSU($nsu, $dtCredito, $vrBrutVenda) {
         $params = array(
            ":DTCREDITO"    => $dtCredito,
            ":NRORG"        => $this->conn->getNRORG(),
            ":VRBRUTVENDA"  => $vrBrutVenda,
            ":CDNSU"        => $nsu,
         );
         
         return $this->conn->fetchRow(self::GET_PAYMENT_BY_NSU, $params, [], true);
    }
    
    private function getPaymentByNRAUTORIZACAO($cdLoteVenda, $autorizacao, $dtCredito, $vrBrutVenda) {
         $params = array(
            ":DTCREDITO"    => $dtCredito,
            ":NRORG"        => $this->conn->getNRORG(),
            ":VRBRUTVENDA"  => $vrBrutVenda,
            ":NRAUTORIZACAO"  => $autorizacao,
            ":CDLOTEVENDA"  => $cdLoteVenda,
         );
         
         return $this->conn->fetchRow(self::GET_PAYMENT_BY_NRAUTORIZACAO, $params, [], true);
    }

    private function getPaymentByCDLOTEVENDA($cdLoteVenda, $dtCredito, $vrBrutVenda) {
         $params = array(
            ":DTCREDITO"    => $dtCredito,
            ":NRORG"        => $this->conn->getNRORG(),
            ":VRBRUTVENDA"  => $vrBrutVenda,
            ":CDLOTEVENDA"  => $cdLoteVenda,
         );
         
         return $this->conn->fetchRow(self::GET_PAYMENT_BY_CDLOTEVENDA, $params, [], true);
    }

    private const GET_PAYMENT_DEFAULT = "
        SELECT COUNT(*) LINHAS
          FROM RETORIGFINANC
         WHERE CDNSU = ':CDNSU'
           AND NRAUTORIZACAO = ':NRAUTORIZACAO' 
           AND DTCREDITO = ':DTCREDITO' 
           AND NRORG = :NRORG
    ";
    
    private const GET_PAYMENT_BY_CDLOTEVENDA = "
        SELECT COUNT(*) LINHAS 
          FROM RETORIGFINANC 
         WHERE CDNSU IS NULL 
           AND CDLOTEVENDA = ':CDLOTEVENDA'
           AND NRAUTORIZACAO IS NULL 
           AND DTCREDITO = ':DTCREDITO' 
           AND NRORG = :NRORG 
           AND VRBRUTVENDA = :VRBRUTVENDA 
    ";
    
    private const GET_PAYMENT_BY_NRAUTORIZACAO = "
        SELECT COUNT(*) LINHAS 
          FROM RETORIGFINANC 
         WHERE CDNSU IS NULL 
           AND CDLOTEVENDA = ':CDLOTEVENDA'
           AND NRAUTORIZACAO = ':NRAUTORIZACAO' 
           AND DTCREDITO = ':DTCREDITO'
           AND NRORG = :NRORG 
           AND VRBRUTVENDA = :VRBRUTVENDA 
    ";
    
    private const GET_PAYMENT_BY_NSU = "
        SELECT COUNT(*) LINHAS 
          FROM RETORIGFINANC 
         WHERE CDNSU = :CDNSU 
           AND NRAUTORIZACAO IS NULL 
           AND CDLOTEVENDA IS NULL 
           AND DTCREDITO = ':DTCREDITO' 
           AND NRORG = :NRORG 
           AND VRBRUTVENDA = :VRBRUTVENDA
    ";
    
    private const INSERT_PAYMENT = "
        INSERT INTO RETORIGFINANC (
            NRRETORIGFINANC,
            NRCTRLACESWS,
            NRESTABELEC,
            NMADQUIRENTE,
            NRADQUIRENTE,
            CDPRODUTOADQ,
            NMPRODUTOADQ,
            NRPRODUTOADQ,
            DTVENDA,
            DTCREDITO,
            CDLOTEVENDA,
            NRAUTORIZACAO,
            CDNSU,
            NRTID,
            VRBRUTVENDA,
            VRTAXAADM,
            VRCOBRADOTAXA,
            VRLIQVENDA,
            CDBANCO,
            CDAGENCIA,
            CDCONTCORR,
            NRPARCELA,
            NRQTPARCELA,
            IDANTECIPADO,
            IDTIPOTRANS,
            DSTRANSACAO,
            NRORG,
            IDATIVO,
            DTINCLUSAO,
            DTULTATU,
            NRORGINCLUSAO,
            NRORGULTATU 
        ) VALUES (
            ':NRRETORIGFINANC',
            ':NRCTRLACESWS',
            ':Estabelecimento',
            UPPER(':Adquirente'),
            ':NRADQUIRENTE',
            ':CDPRODUTOADQ',
            ':TipoProduto',
            ':NRPRODUTOADQ',
            TO_DATE(':DataVenda', 'DD/MM/YYYY'),
            TO_DATE(':DataPagamento', 'DD/MM/YYYY'),
            ':ResumoVenda',
            ':Autorizacao',
            ':NSU',
            ':TID',
            TO_NUMBER(':ValorBruto'),
            TO_NUMBER(':Taxa'),
            ROUND(TO_NUMBER(':VRCOBRADOTAXA'), 2),
            ROUND(TO_NUMBER(':VRLIQVENDA'), 2),
            ':Banco',
            ':Agencia',
            ':Conta',
            ':Parcela',
            ':TotalParcelas',
            ':Antecipacao',
            ':IdTipoTransacao',
            ':TipoTransacao',
            ':NRORG',
            ':IDATIVO',
            TO_DATE(':DTINCLUSAO', 'DD/MM/YYYY HH24:MI:SS'),
            TO_DATE(':DTULTATU', 'DD/MM/YYYY HH24:MI:SS'),
            ':NRORG',
            ':NRORG' )";
}     
    